#!/usr/bin/env bun
import fs from "node:fs";

const supportedTags = [
  "a",
  "abbr",
  "address",
  "article",
  "b",
  "bdi",
  "blockquote",
  "button",
  "caption",
  "cite",
  "code",
  "col",
  "colgroup",
  "dd",
  "del",
  "details",
  "dialog",
  "div",
  "dl",
  "dt",
  "em",
  "fieldset",
  "figcaption",
  "figure",
  "footer",
  "form",
  "h1",
  "h2",
  "h3",
  "h4",
  "h5",
  "h6",
  "header",
  "hr",
  "i",
  "iframe",
  "img",
  "input",
  "ins",
  "kbd",
  "label",
  "legend",
  "li",
  "main",
  "mark",
  "menu",
  "meter",
  "nav",
  "ol",
  "output",
  "p",
  "picture",
  "pre",
  "progress",
  "q",
  "rt",
  "ruby",
  "section",
  "select",
  "span",
  "strong",
  "sub",
  "summary",
  "sup",
  "svg",
  "table",
  "tbody",
  "td",
  "textarea",
  "tfoot",
  "th",
  "thead",
  "time",
  "tr",
  "u",
  "ul",
  "var",
  "video",
  "wbr",
].sort();

function replaceFile(file: string, search: string | RegExp, replace: string) {
  const content = fs.readFileSync(file, "utf-8");
  fs.writeFileSync(file, content.replace(search, replace));
}

replaceFile(
  "src/macro.d.ts",
  /export type SupportedTag =(\n\s+\| "\w+")+;/,
  `export type SupportedTag =\n  | "${supportedTags.join('"\n  | "')}";`
);

replaceFile(
  "src/babel/macro.ts",
  /const supportedTags = new Set<SupportedTag>\(\[(\n\s+"\w+",)+\n]\);/,
  `const supportedTags = new Set<SupportedTag>([\n  "${supportedTags.join('",\n  "')}",\n]);`
);
