import { describe, expect, it } from "vitest";

import { createPostCSS, getClassName } from "../index";

import { getBuild, minCSS, name } from "./utils";

describe("babel-tailwind", () => {
  const compileESBuild = getBuild("base");

  it("supports importing tailwind/base", async () => {
    const postcss = createPostCSS({
      tailwindConfig: {},
    });
    const base = await postcss("@tailwind base;");
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
        import "${name}/base";
      `,
    });

    expect(files.js.text).toBe("");
    expect(minCSS(files.css.text)).toContain(minCSS(base));
  });

  it("supports composeRenderProps", async () => {
    const { files } = await compileESBuild({
      clsx: "clsx",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div css="text-center">
              Hello, world!
            </div>
          );
        }
      `,
    });

    const clsName = getClassName("text-center");
    expect(files.js.text).toContain(`{ className: "${clsName}",`);
  });
});
