import { describe, expect, it } from "vitest";

import { getBuild, matchSnapshot } from "./utils";

describe("options", () => {
  const compileESBuild = getBuild("options");
  it("supports custom jsxAttributeName", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      jsxAttributeName: "tw",
      expectFiles: 2,
      javascript: /* tsx */ `
      export function Hello() {
        return (
          <div tw="text-center">
            Hello, world!
          </div>
        );
      }
    `,
    });
    matchSnapshot(files);
  });

  it("does not remove the attribute if `preserveAttribute` is true", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      jsxAttributeAction: "preserve",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div css="text-center">
              Hello, world!
            </div>
          );
        }
      `,
    });

    expect(files.js.text).toContain(`css: "text-center"`);
  });
});
