/* eslint-disable unicorn/string-content */
import { describe, it } from "vitest";
import { getBuild, matchSnapshot } from "./utils";

describe("spread", () => {
  const compileESBuild = getBuild("spread");

  it('supports spread attribute in "css" attribute', async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
      export function Hello(props) {
        return (
          <div css="text-center" {...props}>
            Hello, world!
          </div>
        );
      }
    `,
    });

    matchSnapshot(files);
  });

  it('supports spread attribute in "css" attribute with an object pattern', async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
      export function Hello({ style, ...props }) {
        return (
          <div css="text-center" {...props}>
            Hello, world!
          </div>
        );
      }
    `,
    });

    matchSnapshot(files);
  });

  it('supports spread attribute in "css" attribute (2)', async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
      export function Hello(props) {
        props = { ...props, className: "text-center" };
        return (
          <div css="text-center" {...props}>
            Hello, world!
          </div>
        );
      }
    `,
    });

    matchSnapshot(files);
  });
});
