import { describe, it } from "vitest";

import { getBuild, matchSnapshot } from "./utils";

describe("babel-tailwind", () => {
  const compileESBuild = getBuild("styleObject");

  it("supports conversion into CSSProperties", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 1,
      javascript: `
        import { tws } from "@aet/tailwind/macro";

        export function Hello() {
          return (
            <div style={tws\`p-2 text-center hover:font-semibold sm:p-1\`}>
              Hello, world!
            </div>
          );
        }
    `,
    });

    matchSnapshot(files, true);
  });

  it("supports .hover, .focus, .active, .group-hover, .group-focus, .group-active", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 1,
      javascript: `
        import { tws } from "@aet/tailwind/macro";

        export const style = [
          tws.hover\`font-semibold\`,
          tws.focus\`font-bold\`,
          tws.active\`font-light\`,
          tws.hover.active\`p-2\`,
        ];
    `,
    });

    matchSnapshot(files, true);
  });
});
