import * as tailwind from "tailwindcss";

import type { ResolveTailwindOptions } from "./index";

declare const __PKG_NAME__: string;
export const pkgName = __PKG_NAME__;

export const macroNames = [`${pkgName}/macro`, `${pkgName}/µ`];
export const utilsName = `${pkgName}/utils`;

interface LineColumn {
  line: number;
  column: number;
}

export interface SourceLocation {
  filename: string;
  start: LineColumn;
  end: LineColumn;
  text: string;
}

export interface StyleMapEntry {
  key: string;
  classNames: string[];
  location: SourceLocation;
}

export const tailwindDirectives = ["components", "utilities", "variants"] as const;

export type StyleMap = Map</* filename */ string, StyleMapEntry[]>;

export function createPostCSS({
  tailwindConfig,
  prefix,
}: Pick<ResolveTailwindOptions, "tailwindConfig" | "prefix">) {
  return async (css: string) => {
    const compiler = await tailwind.compile(
      [prefix, '@config "tailwind.config.js";', css].filter(Boolean).join("\n"),
      {
        // eslint-disable-next-line @typescript-eslint/require-await
        async loadModule(_id, base, resourceHint) {
          if (resourceHint === "config") {
            return { base, module: tailwindConfig };
          }
          throw new Error(`The browser build does not support plugins or config files.`);
        },
      }
    );

    return compiler.build([]).replace(/^\/\*![^*]*\*\/\n/, "");
  };
}

export type Compile = ReturnType<typeof createPostCSS>;
