import postcss from "postcss";
import tailwind from "tailwindcss";

import type { ResolveTailwindOptions } from "./index";

declare const __PKG_NAME__: string;
export const pkgName = __PKG_NAME__;

export const macroNames = [`${pkgName}/macro`, `${pkgName}/µ`];
export const utilsName = `${pkgName}/utils`;

interface LineColumn {
  line: number;
  column: number;
}

export interface SourceLocation {
  filename: string;
  start: LineColumn;
  end: LineColumn;
  text: string;
}

export interface StyleMapEntry {
  key: string;
  classNames: string[];
  location: SourceLocation;
}

export const tailwindDirectives = ["components", "utilities", "variants"] as const;

export type StyleMap = Map</* filename */ string, StyleMapEntry[]>;

export function createPostCSS({
  tailwindConfig,
  prefix,
}: Pick<ResolveTailwindOptions, "tailwindConfig" | "prefix">) {
  const post = postcss([
    tailwind({
      ...tailwindConfig,
      content: [{ raw: "<br>", extension: "html" }],
    }),
  ]);

  return async (css: string) => {
    const result = await post.process((prefix ?? "") + css, { from: undefined });
    return result.css;
  };
}

export type Compile = ReturnType<typeof createPostCSS>;
