import cx from "clsx";
import { type FunctionComponent, forwardRef } from "react";

export interface WithClassName<Props = object> extends FunctionComponent<Props> {
  className: string;
}

type PresetClassNameValue = string | string[];
type PresetClassName<Props = UnknownProps> =
  | PresetClassNameValue
  | ((props: Props) => PresetClassNameValue);

type UnknownProps = Record<string, unknown>;

type InputProps = React.DetailedHTMLProps<
  React.InputHTMLAttributes<HTMLInputElement>,
  HTMLInputElement
>;

export const classed: {
  (
    type: "input",
    className: PresetClassName<InputProps>,
    defaultProps?: Partial<InputProps>
  ): React.FunctionComponent<InputProps>;
  <K extends keyof React.JSX.IntrinsicElements>(
    type: K,
    className: PresetClassName<React.JSX.IntrinsicElements[K]>,
    defaultProps?: Partial<React.JSX.IntrinsicElements[K]>
  ): React.FunctionComponent<React.JSX.IntrinsicElements[K]>;
  (
    type: string,
    className: PresetClassName,
    defaultProps?: UnknownProps
  ): WithClassName<React.ClassAttributes<any> & React.DOMAttributes<any>>;
  <Props>(
    type: FunctionComponent<Props>,
    className: PresetClassName<Props>,
    defaultProps?: Partial<Props>
  ): WithClassName<Props>;
  <Props>(
    type: React.ComponentClass<Props>,
    className: PresetClassName<Props>,
    defaultProps?: Partial<Props>
  ): WithClassName<Props>;
} = (
  Component: any,
  classNameInput: PresetClassName<any>,
  defaultProps?: UnknownProps
) => {
  const className =
    typeof classNameInput === "function"
      ? (props: any) => cx(classNameInput(props))
      : () => cx(classNameInput);
  const component: any = forwardRef<any, any>(({ className: cls, ...props }, ref) => (
    <Component
      {...defaultProps}
      {...props}
      ref={ref}
      className={
        typeof cls === "function"
          ? (...args: unknown[]) => cx(className(props), cls(...args))
          : cx(className(props), cls)
      }
    />
  ));
  component.className = className;
  return component;
};

export function composeClassName<T>(
  value: string,
  prop: string | ((values: T) => string) | undefined
) {
  return typeof prop === "function" ? (arg: T) => cx(value, prop(arg)) : cx(value, prop);
}
