// https://github.com/tailwindlabs/tailwindcss-aspect-ratio/commit/b2a9d02229946f3430c0013198be2affa7a175da
import plugin from "tailwindcss/plugin.js";

const baseStyles = {
  position: "relative",
  paddingBottom: `calc(var(--tw-aspect-h) / var(--tw-aspect-w) * 100%)`,
};

const childStyles = {
  position: "absolute",
  height: "100%",
  width: "100%",
  top: "0",
  right: "0",
  bottom: "0",
  left: "0",
};

const noneComponent = {
  ".aspect-none": {
    position: "static",
    paddingBottom: "0",
  },
  ".aspect-none > *": {
    position: "static",
    height: "auto",
    width: "auto",
    top: "auto",
    right: "auto",
    bottom: "auto",
    left: "auto",
  },
};

export default plugin(
  ({ addComponents, matchComponents, theme }) => {
    const values = theme("aspectRatio")!;

    matchComponents<string>(
      {
        "aspect-w": value => ({
          ...baseStyles,
          "--tw-aspect-w": value,
          "> *": childStyles,
        }),
        "aspect-h": value => ({ "--tw-aspect-h": value }),
      },
      { values }
    );

    addComponents(noneComponent);
  },
  {
    theme: {
      aspectRatio: {
        1: "1",
        2: "2",
        3: "3",
        4: "4",
        5: "5",
        6: "6",
        7: "7",
        8: "8",
        9: "9",
        10: "10",
        11: "11",
        12: "12",
        13: "13",
        14: "14",
        15: "15",
        16: "16",
      },
    },
    variants: {
      aspectRatio: ["responsive"],
    },
  }
);
